package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.TaVisitingLogMapper;
import com.huiju.welcome.model.TaDevice;
import com.huiju.welcome.model.TaPerson;
import com.huiju.welcome.model.TaSnapshotPerson;
import com.huiju.welcome.model.TaVisitingLog;
import com.huiju.welcome.service.ITaDeviceService;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.service.ITaVisitingLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@Service
public class TaVisitingLogServiceImpl extends ServiceImpl<TaVisitingLogMapper, TaVisitingLog> implements ITaVisitingLogService {

    @Autowired
    private TaVisitingLogMapper taVisitingLogMapper;

    @Autowired
    ITaDeviceService iTaDeviceService;

    @Autowired
    ITaPersonService iTaPersonService;

    public List<Map<String, String>> getVisitingDailyAmount(LocalDateTime startDate, LocalDateTime endDate) {
        return taVisitingLogMapper.getVisitingDailyAmount(startDate, endDate);
    }

    @Override
    public Long getRegularClientTodayAmount() {
        List<Map<String, Long>> res = taVisitingLogMapper.getRegularClientTodayAmount();
        if (null == res || res.size() == 0) return 0l;

        return res.get(0).get("total");
    }

    @Override
    public IPage<TaVisitingLog> getVisitingList(Page<TaVisitingLog> page, String startDate, String endDate, Integer personId) {
        return taVisitingLogMapper.getVisitingList(page, startDate, endDate, personId);
    }

    @Override
    public ResponseBean getTimeline() {
        LocalDateTime now = LocalDateTime.now();

        // 四小时内
        LocalDateTime fourHour = now.minusHours(4);
        QueryWrapper<TaVisitingLog> query4 = new QueryWrapper<>();
        query4.ge("visite_date", fourHour);
        query4.groupBy("real_person");
        query4.orderByDesc("visite_date");
        List<TaVisitingLog> fourHourList = taVisitingLogMapper.selectList(query4);

        return ResponseBean.success(fourHourList);
    }

    @Override
    public TaVisitingLog newLogByDevice(TaSnapshotPerson taSnapshotPerson, TaPerson taPerson) {
        TaVisitingLog taVisitingLog = new TaVisitingLog();
        TaDevice taDevice = iTaDeviceService.getById(taSnapshotPerson.getDeviceId());

        taVisitingLog.setPersonId(taPerson.getPersonId());
        taVisitingLog.setRealPerson(taPerson.getRealId());
        taVisitingLog.setAvatar(taPerson.getAvatar());
        taVisitingLog.setDeviceId(taDevice.getDeviceId());
        taVisitingLog.setDeviceName(taDevice.getDeviceName());
        taVisitingLog.setVisiteDate(LocalDateTime.now());
        taVisitingLog.setName(taPerson.getName());

        // 反更新最后来访时间
        taPerson.setLastVisiteDate(taVisitingLog.getVisiteDate());
        taPerson.setLastVisiteDevice(taDevice.getDeviceId());
        UpdateWrapper<TaPerson> wrapper = new UpdateWrapper<>();
        wrapper.set("last_visite_date", taVisitingLog.getVisiteDate());
        wrapper.set("last_visite_device", taDevice.getDeviceId());
        wrapper.eq("person_id", taPerson.getPersonId());
        iTaPersonService.update(taPerson, wrapper);

        int row = taVisitingLogMapper.insert(taVisitingLog);
        return row < 1 ? null : taVisitingLog;
    }
}
